package erpordermanage

import (
	"encoding/json"
	"errors"
	"github.com/gin-gonic/gin"
	model "go-admin/app/admin/models"
	orm "go-admin/common/global"
	"go-admin/logger"
	"go-admin/tools"
	"go-admin/tools/app"
	"net/http"
	"time"
)

// ErpOrderCreate 新建零售订单
// @Summary 新建零售订单
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderCreateReq true "新建零售订单模型"
// @Success 200 {object} app.Response
// @Router /api/v1/erp_order/create [post]
func ErpOrderCreate(c *gin.Context) {
	var req = new(model.ErpOrderCreateReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	sysUser, err := model.GetSysUserByCtx(c)
	if err != nil {
		logger.Error("sys user err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "操作失败:"+err.Error())
		return
	}

	if sysUser.StoreData == "" { // 校验登陆账户所属门店
		logger.Error("sys user store id null")
		app.Error(c, http.StatusBadRequest, errors.New("para err"), "该账户未绑定门店")
		return
	}

	err = model.CreateErpOrder(req, sysUser)
	if err != nil {
		logger.Error("CreateErpOrder err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	app.OK(c, nil, "")
	return
}

// ErpOrderEdit 编辑零售订单
// @Summary 编辑零售订单
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderCreateReq true "编辑零售订单模型"
// @Success 200 {object} app.Response
// @Router /api/v1/erp_order/edit [post]
func ErpOrderEdit(c *gin.Context) {
	var req = new(model.ErpOrderCreateReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	sysUser, err := model.GetSysUserByCtx(c)
	if err != nil {
		logger.Error("sys user err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "操作失败:"+err.Error())
		return
	}

	if sysUser.StoreData == "" { // 校验登陆账户所属门店
		logger.Error("sys user store id null")
		app.Error(c, http.StatusBadRequest, errors.New("para err"), "该账户未绑定门店")
		return
	}

	err = model.EditErpOrder(req, sysUser)
	if err != nil {
		logger.Error("EditErpOrder err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	app.OK(c, nil, "")
	return
}

// ErpOrderList 查询零售订单列表
// @Summary 查询零售订单列表
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderListReq true "查询零售订单列表模型"
// @Success 200 {object} models.ErpOrderListResp
// @Router /api/v1/erp_order/list [post]
func ErpOrderList(c *gin.Context) {
	req := &model.ErpOrderListReq{}
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	resp, err := req.List()
	if err != nil {
		logger.Error("erp commodity list err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "查询失败:"+err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}

// ErpOrderAudit 审核零售订单（审核、取消审核）
// @Summary 审核零售订单
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderAuditReq true "审核零售订单模型"
// @Success 200 {object} app.Response
// @Router /api/v1/erp_order/audit [post]
func ErpOrderAudit(c *gin.Context) {
	var req = new(model.ErpOrderAuditReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}
	var erpOrder model.ErpOrder
	err = orm.Eloquent.Table("erp_order").Where("bill_sn = ?", req.BillSn).Find(&erpOrder).Error
	if err != nil {
		logger.Error("order err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "审核失败:"+err.Error())
		return
	}

	orderState := model.ErpOrderStateAudited
	stockState := model.OnSale
	switch req.State {
	case 1: // 审核
		orderState = model.ErpOrderStateAudited
		stockState = model.OnSale // 库存-销售锁定
	case 2: // 取消审核
		orderState = model.ErpOrderStateUnAudit
		stockState = model.InStock // 库存-在库
	default:
		logger.Error("req.State no in (1,2)", logger.Field("req.State", req.State))
		app.Error(c, http.StatusBadRequest, errors.New("para err"), "参数错误,req.State no in (1,2)")
		return
	}

	// 检查支付方式，如果没有线上支付则默认为支付已完成
	nPayStatus := model.WaitForPaying // 待支付
	var cashiers []model.ErpOrderCashier
	err = json.Unmarshal([]byte(erpOrder.CashierList), &cashiers)
	if err != nil {
		logger.Error("unmarshal CashierList err:", logger.Field("err", err))
	}
	if !model.CheckIsOnlinePay(cashiers) {
		nPayStatus = model.HavePaid // 已完成
		stockState = model.SoldOut  // 库存-已售
	}

	begin := orm.Eloquent.Begin()
	err = begin.Table("erp_order").Where("id = ?", erpOrder.ID).Updates(map[string]interface{}{
		"state":      orderState,
		"pay_status": nPayStatus,
		"audit_time": time.Now(),
	}).Error
	if err != nil {
		begin.Rollback()
		logger.Error("order err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "审核失败:"+err.Error())
		return
	}

	// 更新库存
	err = model.UpdateStock(begin, erpOrder, stockState)
	if err != nil {
		begin.Rollback()
		logger.Error("UpdateStock err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "审核失败:"+err.Error())
		return
	}

	err = begin.Commit().Error
	if err != nil {
		begin.Rollback()
		logger.Error("UpdateStock err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "审核失败:"+err.Error())
		return
	}

	app.OK(c, nil, "")
	return
}

// ErpOrderDelete 删除零售订单
// @Summary 删除零售订单
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderDeleteReq true "删除零售订单模型"
// @Success 200 {object} app.Response
// @Router /api/v1/erp_order/delete [post]
func ErpOrderDelete(c *gin.Context) {
	var req = new(model.ErpOrderDeleteReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	err = model.DeleteOrder(req)
	if err != nil {
		logger.Error("AddInvoice err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	app.OK(c, nil, "")
	return
}

// ErpOrderAddInvoice 开发票
// @Summary 开发票
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderAddInvoiceReq true "开发票模型"
// @Success 200 {object} app.Response
// @Router /api/v1/erp_order/add_invoice [post]
func ErpOrderAddInvoice(c *gin.Context) {
	var req = new(model.ErpOrderAddInvoiceReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	err = model.SetInvoice(req)
	if err != nil {
		logger.Error("AddInvoice err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "操作失败:"+err.Error())
		return
	}

	app.OK(c, nil, "")
	return
}

// ErpOrderPay 收款
// @Summary 收款
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderPayReq true "收款模型"
// @Success 200 {object} models.ErpOrderPayResp
// @Router /api/v1/erp_order/pay [post]
func ErpOrderPay(c *gin.Context) {
	var req = new(model.ErpOrderPayReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	resp, err := model.ErpOrderPay(req)
	if err != nil {
		logger.Error("ErpOrderPay err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}

// ErpOrderQueryPayStatus 查询付款状态
// @Summary 查询付款状态
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderDeleteReq true "查询付款状态模型"
// @Success 200 {object} models.ErpOrderPayResp
// @Router /api/v1/erp_order/query_pay_status [post]
func ErpOrderQueryPayStatus(c *gin.Context) {
	var req = new(model.ErpOrderDeleteReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	resp, err := model.QueryErpOrderPayStatus(req.BillSn)
	if err != nil {
		logger.Error("QueryErpOrderPayStatus err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "查询失败:"+err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}

// ErpOrderStoreManageData 查询门店经营数据
// @Summary 查询门店经营数据
// @Tags 零售报表
// @Produce json
// @Accept json
// @Param request body models.ErpOrderStoreManageDataReq true "查询门店经营数据模型"
// @Success 200 {object} models.ErpOrderStoreManageDataResp
// @Router /api/v1/erp_order/store_manage_data [post]
func ErpOrderStoreManageData(c *gin.Context) {
	var req = new(model.ErpOrderStoreManageDataReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	resp, err := model.QueryStoreManageData(req)
	if err != nil {
		logger.Error("QueryStoreManageData err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "查询失败:"+err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}

// ErpOrderRetailMargin 查询商品零售毛利汇总
// @Summary 查询商品零售毛利汇总
// @Tags 零售报表
// @Produce json
// @Accept json
// @Param request body models.ErpOrderRetailMarginReq true "查询商品零售毛利汇总数据模型"
// @Success 200 {object} models.ErpOrderRetailMarginResp
// @Router /api/v1/erp_order/retail_margin [post]
func ErpOrderRetailMargin(c *gin.Context) {
	var req = new(model.ErpOrderRetailMarginReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	resp, err := model.QueryRetailMargin(req)
	if err != nil {
		logger.Error("QueryRetailMargin err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "查询失败:"+err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}

// ErpOrderRetailDetail 查询零售明细
// @Summary 查询零售明细
// @Tags 零售报表
// @Produce json
// @Accept json
// @Param request body models.ErpOrderRetailDetailReq true "查询零售明细数据模型"
// @Success 200 {object} models.ErpOrderRetailDetailResp
// @Router /api/v1/erp_order/retail_detail [post]
func ErpOrderRetailDetail(c *gin.Context) {
	var req = new(model.ErpOrderRetailDetailReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	resp, err := model.QueryRetailDetail(req)
	if err != nil {
		logger.Error("QueryRetailDetail err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "查询失败:"+err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}

// ErpOrderReceiptData 查询小票数据
// @Summary 查询小票数据
// @Tags 零售订单
// @Produce json
// @Accept json
// @Param request body models.ErpOrderDeleteReq true "查询小票数据模型"
// @Success 200 {object} models.ErpOrderReceiptDataResp
// @Router /api/v1/erp_order/receipt_data [post]
func ErpOrderReceiptData(c *gin.Context) {
	var req = new(model.ErpOrderDeleteReq)
	if err := c.ShouldBindJSON(&req); err != nil {
		logger.Error("ShouldBindJSON err:", logger.Field("err", err))
		app.Error(c, http.StatusBadRequest, err, "参数错误:"+err.Error())
		return
	}

	err := tools.Validate(req) //必填参数校验
	if err != nil {
		app.Error(c, http.StatusBadRequest, err, err.Error())
		return
	}

	resp, err := model.QueryReceiptData(req)
	if err != nil {
		logger.Error("QueryRetailDetail err:", logger.Field("err", err))
		app.Error(c, http.StatusInternalServerError, err, "查询失败:"+err.Error())
		return
	}

	app.OK(c, resp, "")
	return
}
